@@ -45,7 +45,9 @@ strtoint64(pTHX_ const char *s, int base, int sign)
 	uint64_t acc = 0;
 	int c, neg, between = 0;
 
-        uint64_t upper_mul_limit;
+        int mdoo = may_die_on_overflow; /* we copy it to avoid a race
+                                           condition */
+        uint64_t upper_mul_limit = 0;
 
 	/*
 	 * Skip white space and pick up leading +/- sign if any.
@@ -73,7 +75,7 @@ strtoint64(pTHX_ const char *s, int base, int sign)
 	if (base == 0)
 		base = c == '0' ? 8 : 10;
 
-        if (may_die_on_overflow) upper_mul_limit = UINT64_MAX / base;
+        if (mdoo) upper_mul_limit = UINT64_MAX / base;
 
         for (;; c = (unsigned char) *s++) {
                 if (isdigit(c))
@@ -86,7 +88,7 @@ strtoint64(pTHX_ const char *s, int base, int sign)
 			break;
                 if (c >= base)
 			break;
-                if (may_die_on_overflow) {
+                if (mdoo) {
                     if (acc > upper_mul_limit) overflow(aTHX_ (sign ? out_of_bounds_error_s : out_of_bounds_error_u));
                     acc *= base;
                     if (UINT64_MAX - acc < c) overflow(aTHX_ (sign ? out_of_bounds_error_s : out_of_bounds_error_u));
@@ -97,7 +99,7 @@ strtoint64(pTHX_ const char *s, int base, int sign)
                 }
                 between = 1;
         }
-        if ( may_die_on_overflow && sign &&
+        if ( mdoo && sign &&
              ( acc > (neg ? (~(uint64_t)INT64_MIN + 1) : INT64_MAX) ) ) overflow(aTHX_ out_of_bounds_error_s);
 
         return (neg ? ~acc + 1 : acc);
